(* $Id: ugraphics.mli,v 1.9 2014/02/27 16:36:36 fclement Exp $ *)

open Unatural;;

type coordinate = natural
(** The type of coordinates for the [Ugraphics] library. *)
;;

type abscissa = coordinate
and ordinate = coordinate
(** The types of abscissa and ordinate of a pixel. *)
;;

type center_abscissa = abscissa
and center_ordinate = ordinate
(** The types of abscissa and ordinate of the center of a ball. *)
;;

type length = natural
(** The type of nonnegative length of graphical entities. *)
;;

type width = length
and height = length
(** The types of width and height of graphical entities. *)
;;

type aspect_ratio = width * height
(** The type of rational ratios. *)
;;

type radius = length
(** The type of radius of graphical entities. *)
;;

type principal_radius = radius
and secondary_radius = radius
(** The types of radius of graphical entities. *)
;;

type angle = int
(** *)
;;

type from_angle = angle
and to_angle = angle
(** The types of angles of graphical entities. *)
;;

type color = int32
(** *)
;;

type color_channel = char
(** *)
;;

type alpha_channel = color_channel
and red_channel = color_channel
and green_channel = color_channel
and blue_channel = color_channel
(** *)
;;

type from_color = color
and to_color = color
(** The types of colors of graphical entities. *)
;;

val aspect_ratio_of_int_pair : int * int -> aspect_ratio;;
val int_pair_of_aspect_ratio : aspect_ratio -> int * int;;

val color_channel_of_natural : natural -> color_channel;;
val natural_of_color_channel : color_channel -> natural;;

val r_of : color -> red_channel;;
val g_of : color -> green_channel;;
val b_of : color -> blue_channel;;
val a_of : color -> alpha_channel;;
(** Decomposition of a color into its basic components. *)

val argb :
  alpha_channel -> red_channel -> green_channel -> blue_channel -> color
(** [argb a r g b] returns the color with the corresponding components. *)
;;

val rgb : red_channel -> green_channel -> blue_channel -> color
(** Return a color with maximal alpha channel *)
;;

val rgb_int : int -> int -> int -> color;;

val rgb_of_color :
  color -> (red_channel * green_channel * blue_channel)
;;

type steps = natural
and max_steps = natural
and from_natural = natural
and to_natural = natural;;

val scale_natural : float -> natural -> natural;;

val round_float : float -> natural;;

val scale : steps -> max_steps -> from_natural -> to_natural -> natural
(**
  Scaling from c to c', by i steps over w steps.
  scale i w c c' = (c' * i + c * (w - i)) / w

  In particular, we have:
  scale 0 w c c' = c           (1)
  scale w w c c' = c'          (2)

*)
;;

val scale_color : steps -> max_steps -> from_color -> to_color -> color
(** *)
;;

val fill_rectangle : abscissa -> ordinate -> width -> height -> unit
(** [fill_rectangle x y w h] fills the rectangle with lower left corner at
  coordinates ([x], [y]), width [w] and height [h], with the current color.
  If [w] or [h] is null, the rectangle is empty, hence nothing is drawn.
  If [w] and [h] are one, the rectangle has only one pixel.

  Raise Invalid_argument if w or h is negative. *)
;;

val fill_ball_inf :
  center_abscissa -> center_ordinate ->
  principal_radius -> secondary_radius -> unit
(** To fill balls associated with the L_infinite norm, i.e. the sup norm:
  ||u||_sup = max |u_i|. *)
;;

val fill_rectangle_ball :
  center_abscissa -> center_ordinate ->
  principal_radius -> secondary_radius -> unit
(** A synonym for [fill_ball_inf]. *)
;;

val fill_ball_euclidean :
  center_abscissa -> center_ordinate ->
  principal_radius -> secondary_radius -> unit
(** To fill balls associated with the L_2 norm, i.e. the Euclidean norm:
  ||u||_2 = (sum_i |u_i|^2)^(1/2). *)
;;

val fill_ellipse_ball :
  center_abscissa -> center_ordinate ->
  principal_radius -> secondary_radius -> unit
(** A synonym for [fill_ball_euclidean] . *)
;;

val fill_inscripted_ellipse :
  abscissa -> ordinate -> principal_radius -> secondary_radius -> unit
(**  *)
;;

val fill_rect : abscissa -> ordinate -> width -> height -> unit
(** *)
;;
val fill_ellipse :
  abscissa -> ordinate -> principal_radius -> secondary_radius -> unit
(** *)
;;

type x = abscissa
and y = ordinate
and xc = center_abscissa
and yc = center_ordinate
and w = width
and h = height
and r = radius
and rx = principal_radius
and ry = secondary_radius
(** Convenient abbreviations. *)
;;

val set_color : color -> unit
;;
val moveto : x -> y -> unit
;;
val lineto : x -> y -> unit
;;

val set_line_width : width -> unit
;;

val draw_circle : xc -> yc -> r -> unit
;;

val draw_arc : xc -> yc -> rx -> ry -> from_angle -> to_angle -> unit
;;

val plot : x -> y -> unit
;;

val open_graph : string -> unit;;

val fill_poly : (x * y) array -> unit;;

val fill_circle : xc -> yc -> radius -> unit;;

val red : color
val green : color
val blue : color
val white : color
val black : color
val yellow : color
val cyan : color
val magenta : color
;;
